var path = require('path');
var fs = require('fs');
var fse = require('fs-extra');
var util = require('./util');
var Page = require('./page');
var AppWxss = require('./app_wxss');
var AppAdminWxss = require('./app_admin_wxss');

var chokidar = require("chokidar");
var debug = require('debug')('wevue');

var Backend = require('./backend');
var BackendConfig = require('./backend_config');
var BackendWxss = require('./app_admin_wxss');
var CreatePage = require('./create_page');
var Plugin = require('./plugin');
var exec = require('./exec');
//加入wath功能，
//如果是wxml，转化后copy到目标目录
//如果是wxss(非app.wxss),转化后copy到目标目录
//其它文件直接copy到目标目录


module.exports = function(folder, vueFolder, rootFolder, wevueFolder, program) {

  debug('wath folder', folder);
  debug('rootFolder', rootFolder);

  var watcher = chokidar.watch(folder, {
    ignored: /(^|[\/\\])\../,
    persistent: true
  });

  watcher.on('add', function(path) {
      debug(`File ${path} has been added`);
      process(path, 'add');
    })
    .on('change', function(path) {
      console.log(`File ${path} has been changed`);
      process(path, 'chage');
    })
    .on('unlink', function(path) {
      console.log(`File ${path} has been removed`)
    });

  function process(file, action) {
    var rPath = file.replace(folder, '');
    debug('rPath', rPath);
    var stat = fs.statSync(file);
    debug('==> isFile?', stat.isFile());
    if (!stat.isFile()) {
      return;
    }

    if ((/\.wxml$/.test(file) || /\.wxss$/.test(file))) {
      //生成新的.vue文件

      if (rPath === '/global.wxss' || rPath === '\\global.wxss') {

        debug('global.wxss --> global.css');
        AppWxss(folder, vueFolder, rootFolder);

      } else if (rPath === '/app.wxss' || rPath === '\\app.wxss') {
        //生成backend.vue
        debug('app.wxss --> app.vue');
        Backend(folder, vueFolder, rootFolder, wevueFolder, program);

      } else if (rPath === '/app.wxml' || rPath === '\\app.wxml') {

        debug('app.wxml --> app.vue');
        Backend(folder, vueFolder, rootFolder, wevueFolder, program);

      } else {
        debug('wx page  --> vue page');
        var pathInfo = path.parse(rPath);

        debug(pathInfo.dir);
        debug(pathInfo.name);

        var page = path.join(pathInfo.dir, pathInfo.name);
        Page(folder, vueFolder, page, rootFolder, wevueFolder, program, function(err, res) {
          if (err) {
            console.log('err', err);
          }
        });
      }

    } else {
      var dest = path.join(vueFolder, 'src', rPath);
      debug('source', file);
      debug('desc', dest);

      //如果是文件夹，就创建一个



      var pathInfo = path.parse(dest);
      fse.mkdirsSync(pathInfo.dir);

      var readerStream = fs.createReadStream(file);
      var writerStream = fs.createWriteStream(dest);
      readerStream.pipe(writerStream);

      if (rPath === '/app.json' || rPath === '\\app.json' || rPath === '/app-admin.json' || rPath === '\\app-admin.json') {
        debug("检查有没有增加新的路由");
        return CreatePage(folder, wevueFolder, file, program)
      } else if (/\.json$/.test(rPath) || /\.ddep\.js$/.test(rPath) || /\.cdep\.js$/.test(rPath)) {
        pathInfo = path.parse(rPath);

        debug(pathInfo.dir);
        debug(pathInfo.name);
        var page = path.join(pathInfo.dir, pathInfo.name);
        Page(folder, vueFolder, page, rootFolder, wevueFolder, program, function(err, res) {
          if (err) {
            console.log('err', err);
          }
        });
      }

      if (action == 'change' && (rPath === '/plugin.json' || rPath === '\\plugin.json')) {
        debug("安装或者更新包");

        var cmd = Plugin.getInstallCmd(folder);
        cmd = `cd ${program.output} && ${cmd}`;
        if (program.registry){
          cmd = `${cmd} --registry=${program.registry}`;
        }
        debug('cmd', cmd);
        exec(cmd, function(err, stdout, stderr) {
          if (err) {
            console.log(err);
          } else {
            debug('install success');
          }

        })
      }
    }
  }
}
